@@ -81,6 +81,8 @@ sub new {
 	 my @cols = $self->{dman}->get_field_names ;
 	 # cols holds the field names from the table. Nothing else !
  	$self->{cols} = \@cols;
+	my %hcols =  map {$_ => 1} @cols;
+        $self->{hcols} = \%hcols;
 	$self->{log}->debug("cols: ". join(" ", @cols));
 	
 	$self->_setup_fields;
@@ -245,7 +247,8 @@ sub _setup_treeview {
 		else	{
 			$self->{log}->debug("field name with txt renderer: " . $field->{name} );
 			
-			if ($field->{name} ~~ @apk) {
+			#if ($field->{name} ~~ @apk) {
+			if ( grep /^$field->{name}$/, @apk ) {
 				$self->{log}->debug("not editable because it's a pk");
 				$renderer->set( editable => FALSE );
 			} else {
@@ -445,7 +448,8 @@ sub _setup_combo {
 		              
                 	} else {
 				my $x = "";
-				if ( $field->{name} ~~ @{$self->{cols}}) {
+				# if ( $field->{name} ~~ @{$self->{cols}}) {
+				if ( defined $self->{hcols}->{$field->{name}} ) {
 					# $self->{log}->debug("query: " .  $field->{name} . " row: " . $i );
 					
 					$x = $self->{dman}->get_field( $field->{name} );
@@ -683,7 +687,8 @@ sub apply {
 	 } else {  # changed, inserted
 
 		for my $field ( @{$self->{fields}} ) {
-			if ( $field->{name} ~~ @{$self->{cols}}){
+			# if ( $field->{name} ~~ @{$self->{cols}}){
+			if ( defined $self->{hcols}->{$field->{name}} ) {
 				my $x =  $model->get( $iter, $self->{colname_to_number}->{ $field->{name} } );
 				$self->{log}->debug("Field: "  . $field->{name} . " row_pos " . $row_pos . " value: ". ($x?$x: "undef") );
 				$self->{dman}->set_field($field->{name}, $x );
