@@ -104,7 +104,7 @@ sub new {
          my %hdates = map {$_ => 1} @dates;
          $self->{hdates_formatted} = \%hdates;
  	$self->{dates_formatters} = {};
-
+	$self->{inserting} = 0;
  	$self->{pos2del} = [];
 
 	return $self;
@@ -296,7 +296,12 @@ sub _display_data {
 
 	$self->{pos} = $pos;
 
-	$dman->set_row_pos($pos) unless ($pos<0);
+	#$dman->set_row_pos($pos) unless ($pos<0);
+	if ($pos < 0) {
+		#$dman->first;
+	}else {
+		$dman->set_row_pos($pos);
+	}
 
 	$self->{painting}=1;
 	#foreach my $id (keys %{$self->{datawidgets}}){
@@ -347,7 +352,7 @@ sub _display_data {
    my $first = ($pos < 0 ? 0 : 1);
   
    $self->_set_record_status_label;
-   $self->_set_rs_range($first);
+   $self->_set_rs_range($first,  $self->{dman}->row_count);
    $self->{on_current}() if ($self->{on_current});
    $self->{painting}=0;
    $self->{changed}=0;
@@ -359,6 +364,7 @@ sub undo{
 	my $self = shift;
 	 $self->{log}->debug("undo clicked");
 	 $self->{changed}=0;
+	 $self->{inserting} = 0;
 	 $self->{pos2del}= [];
 	 $self->_display_data( $self->{pos}  );
 
@@ -373,18 +379,23 @@ sub insert {
 	my $self = shift;
 	$self->{log}->debug("insert");
 	# my $row = $self->{data}[0]->new;
-
+	$self->{inserting} = 1;
 	#$self->{pos} = $self->{count} + 1;
 	#afficher des champs vides
-	$self->_display_data(-1);
-	#data_manager->new_row is called when apply is cliked
+ 	my $new_pos = $self->{dman}->row_count;
+	#data_manager->new_row is called when apply is cliked / but defaults value are not displayed then
+	$self->{dman}->new_row;
+	#$self->_display_data(-1);
+	my $first = ($new_pos > 0 ? 1 : 0);
+	# SqlADM and DBIDM ->new_row does not change row_count (are the others DM similar ?)
+	$self->{log}->debug("insert : row count is : ", $self->{dman}->row_count, " new pos is : ", $new_pos);
+	$self->_display_data($new_pos);
 	   if ($self->{rec_spinner}){
 		  #	my $last = $self->{dman}->row_count;
 		$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
-        	$self->{rec_spinner}->set_value(0);
+		$self->{rec_spinner}->set_range($first, $new_pos+1 );
+		$self->{rec_spinner}->set_value($new_pos + 1);
 	        $self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
-		 
-    
     	} 
 
 }
@@ -416,7 +427,7 @@ sub apply{
 	if ($self->{pos}<0){
 		#my $class =  $self->{class};
 		#$row = $class->new;
-		 $self->{dman}->new_row;
+		#$self->{dman}->new_row;
 			#push @{$self->{data}}, $row;
 			 # $self->{count} ++;
 		 $self->{log}->debug("New row");
@@ -433,8 +444,13 @@ sub apply{
 	if (scalar @{$self->{pos2del}}){
 		$self->{pos2del} = [];
 		$self->{changed} = 0;
+		my $last = $self->{dman}->row_count;
 		# $self->set_record_status_label;
-		$self->{rec_spinner}->set_value(1) if ($self->{rec_spinner});
+		if ($last > 0) {
+			$self->{rec_spinner}->set_value(1) if ($self->{rec_spinner});
+		} else {
+			$self->_display_data(-1);
+		}
 		return;
 	}
 	#updating a new or an existing record
@@ -488,8 +504,9 @@ sub apply{
 		$self->{log}->debug($id . " not in data");
 	   }
 	} #foreach
-	#$row->save;
+	$self->{log}->debug("dman->save");
 	$done = $self->{dman}->save;
+	#$self->set_widget_value($pk[0], $self->{dman}->get_field($pk[0]));
 	# $self->{log}->debug("nofm: " . $row->nofm);
 	# $self->set_rs_range;
 
@@ -513,21 +530,25 @@ sub apply{
 		my $coderef = $self->{after_insert};
 		&$coderef(undef, \%pk_val );
 	}
-	if ($done && $self->{pos}<0){
+	#if ($done && $self->{pos}<0){
+	if ($done && $self->{inserting}){
 		my $last = $self->{dman}->row_count -1;
+		$last = ($last < 0 ? 0 : $last);
 		$self->{log}->debug("last is " . $last );
 		$self->_display_data( $last );
+		$self->{inserting} = 0;
 		 if ($self->{rec_spinner}){
 		  #	my $last = $self->{dman}->row_count;
 			$self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
         		$self->{rec_spinner}->set_value($last+1);
 	        	$self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
     		}
-       	} else {
-		$self->{changed}=0;
+       	}# else {
+         #		$self->{changed}=0;
 	
-	}
+	 #}
 	if ($done) {
+		$self->{changed} = 0;
 		$self->_save_subforms;
 
 		$self->_set_record_status_label;
@@ -633,7 +654,7 @@ sub _set_recordspinner {
 
 
 sub _set_rs_range {
-    my ( $self, $first ) = @_;
+    my ( $self, $first, $last ) = @_;
 
     # Convenience function that sets the min / max value of the record spinner
     	$self->{log}->debug("set_rs_range  first : " . $first);
@@ -642,7 +663,7 @@ sub _set_rs_range {
 	    $self->{log}->debug("adj lower : ". $ad->lower);
 	 if ($first < $ad->lower){ $ad->lower($first); $self->{rec_spinner}->set_adjustment( $ad ); }
         $self->{rec_spinner}->signal_handler_block( $self->{rs_value_changed_signal} );
-        $self->{rec_spinner}->set_range( $first, $self->{dman}->row_count );
+        $self->{rec_spinner}->set_range( $first, $last );
         $self->{rec_spinner}->signal_handler_unblock( $self->{rs_value_changed_signal} );
     }
     $self->{rec_count_label}->set_text(" / " . $self->{dman}->row_count);
